﻿/*:
 * @target MZ
 * @plugindesc a plugin that makes the maximum number of party members infinite.
 * @author Malomi
 * @url https://gendaigoyaku.com/
 * 
 * @param maxPartyMember
 * @text maximum number of party members
 * @desc You can have as many party members as you want.
 * @default 128
 * @type number
 * @max 128
 * @min 4
 * 
 * @help
 * ----------------------------------------------------------------------------
 * MaxPartyMember.js
 * ----------------------------------------------------------------------------
 * This plugin allows you to freely change the maximum number of party members.
 * Just enable it, and you’re good to go.
 * The initial setting for the maximum number of party members is 128. You can change it freely.
 * ----------------------------------------------------------------------------
 * License:
 * This is the MIT License. 
 * It can be used for both commercial and non-commercial purposes.
 * ----------------------------------------------------------------------------
 */

/*:ja
 * @target MZ
 * @plugindesc パーティーの最大人数が無限大になるプラグイン
 * @author まろみ
 * @url https://gendaigoyaku.com/
 *
 * @param maxPartyMember
 * @text パーティーの最大人数
 * @desc 初期設定は128です。好きに変更してね。
 * @default 128
 * @type number
 * @max 128
 * @min 4
 * 
 * @help
 * ----------------------------------------------------------------------------
 * MaxPartyMember.js
 * ----------------------------------------------------------------------------
 * このプラグインは、パーティーの最大人数を自由に変更できるようにするよ。
 * 特に設定もいらないよ。プラグインを有効化するだけでOKです。
 * 最大人数の初期設定は128。自由に変更可能だよ。
 * ----------------------------------------------------------------------------
 * 利用規約:
 * 商用・非商用問わず利用可能です。
 * ----------------------------------------------------------------------------
 */

(() => {
    'use strict';

    const parameters = PluginManager.parameters('MaxPartyMember');
    const maxPartyMembers = Number(parameters['maxPartyMember'] || 128);

    Game_Party.prototype.maxBattleMembers = function () {
        return maxPartyMembers;
    };

    Scene_Battle.prototype.createStatusWindow = function () {
        const rect = this.statusWindowRect();
        const statusWindow = new Window_BattleStatus(rect);
        this.addWindow(statusWindow);
        this._statusWindow = statusWindow;
    };

    Scene_Battle.prototype.statusWindowRect = function () {
        const extra = 10;
        const ww = Graphics.boxWidth - 192;
        const wh = this.windowAreaHeight() + extra;
        const wx = this.isRightInputMode() ? 0 : Graphics.boxWidth - ww;
        const wy = Graphics.boxHeight - wh + extra - 4;
        return new Rectangle(wx, wy, ww, wh);
    };

    const _Window_BattleStatus_initialize = Window_BattleStatus.prototype.initialize;
    Window_BattleStatus.prototype.initialize = function (rect) {
        _Window_BattleStatus_initialize.call(this, rect);
    };

    Window_BattleStatus.prototype.calculateRows = function () {
        const partySize = $gameParty.battleMembers().length;
        return Math.ceil(partySize / 4);
    };

    Window_BattleStatus.prototype.calculateScale = function () {
        const rows = this.calculateRows();
        return 1 / rows;
    };

    Window_Base.prototype.calculateScale = function () {
        if (this instanceof Window_BattleStatus) {
            const rows = this.calculateRows();
            return 1 / rows;
        }
        return 1;
    };

    Window_BattleStatus.prototype.maxCols = function () {
        return 4;
    };

    Window_BattleStatus.prototype.itemHeight = function () {
        const scale = this.calculateScale();
        return this.innerHeight * scale;
    };

    Window_BattleStatus.prototype.itemWidth = function () {
        return this.innerWidth / 4;
    };

    Window_BattleStatus.prototype.drawItemImage = function (index) {
        const actor = this.actor(index);
        const rect = this.faceRect(index);
        this.drawActorFace(actor, rect.x, rect.y, rect.width, rect.height);
    };

    Window_Base.prototype.drawFace = function (faceName, faceIndex, x, y, width, height) {
        width = width || ImageManager.faceWidth;
        height = height || ImageManager.faceHeight;
        const bitmap = ImageManager.loadFace(faceName);
        const pw = ImageManager.faceWidth;
        const ph = ImageManager.faceHeight;
        const sw = Math.min(width, pw);
        const sh = Math.min(height, ph);
        const dx = Math.floor(x + Math.max(width - pw, 0) / 2);
        const dy = Math.floor(y + Math.max(height - ph, 0) / 2);
        const sx = Math.floor((faceIndex % 4) * pw + (pw - sw) / 2);
        const sy = Math.floor(Math.floor(faceIndex / 4) * ph + (ph - sh) / 2);
        const scale = this.calculateScale();
        const scaledHeight = sh * scale;
        const offsetY = (sh - scaledHeight) / 2;
        this.contents.blt(bitmap, sx, sy, sw, sh, dx, dy + offsetY, sw, scaledHeight);
    };

    Window_Selectable.prototype.itemRect = function (index) {
        const maxCols = this.maxCols();
        const itemWidth = this.itemWidth();
        const itemHeight = this.itemHeight();
        const colSpacing = this.colSpacing();
        const rowSpacing = this.rowSpacing();
        const col = index % maxCols;
        const row = Math.floor(index / maxCols);
        const x = col * itemWidth + colSpacing / 2 - this.scrollBaseX();
        const y = row * itemHeight + rowSpacing / 2 - this.scrollBaseY();
        const width = itemWidth - colSpacing;
        const height = itemHeight - rowSpacing;
        return new Rectangle(x, y, width, height);
    };

    Window_Selectable.prototype.itemWidth = function () {
        return Math.floor(this.innerWidth / this.maxCols());
    };

    Window_Selectable.prototype.itemHeight = function () {
        return Window_Scrollable.prototype.itemHeight.call(this) + 8;
    };

    Window_Gold.prototype.colSpacing = function () {
        return 0;
    };

    Window_BattleStatus.prototype.rowSpacing = function () {
        return 0;
    };

    Window_StatusBase.prototype.placeActorName = function (actor, x, y) {
        const key = "actor%1-name".format(actor.actorId());
        const sprite = this.createInnerSprite(key, Sprite_Name);
        sprite.setup(actor);
        sprite.move(x, y);
        const scale = this.calculateScale();
        sprite.scale.set(1, scale);
        sprite.show();
    };

    Window_StatusBase.prototype.placeStateIcon = function (actor, x, y) {
        const key = "actor%1-stateIcon".format(actor.actorId());
        const sprite = this.createInnerSprite(key, Sprite_StateIcon);
        sprite.setup(actor);
        sprite.move(x, y);
        const scale = this.calculateScale();
        sprite.scale.set(1, scale);
        sprite.show();
    };

    Window_StatusBase.prototype.calculateRows = function () {
        const partySize = $gameParty.battleMembers().length;
        return Math.ceil(partySize / 4);
    };

    Window_StatusBase.prototype.gaugeLineHeight = function () {
        if ($gameParty.inBattle()) {
            const rows = this.calculateRows();
            return 24 / rows;
        } else {
            return 24;
        }
    };

    Window_StatusBase.prototype.placeGauge = function (actor, type, x, y) {
        const key = "actor%1-gauge-%2".format(actor.actorId(), type);
        const sprite = this.createInnerSprite(key, Sprite_Gauge);
        sprite.setup(actor, type);
        sprite.move(x, y);
        const scale = this.calculateScale();
        sprite.scale.set(1, scale);
        sprite.show();
    };

    const _BattleManager_initMembers = BattleManager.initMembers;
    BattleManager.initMembers = function () {
        _BattleManager_initMembers.call(this);
    };

    BattleManager.getPlayerFormationRows = function () {
        const partySize = $gameParty.battleMembers().length;
        return Math.ceil(partySize / 4);
    };

    const _Scene_Battle_createSpriteset = Scene_Battle.prototype.createSpriteset;
    Scene_Battle.prototype.createSpriteset = function () {
        _Scene_Battle_createSpriteset.call(this);
        this.setupPlayerFormation();
    };
    Scene_Battle.prototype.setupPlayerFormation = function () {
        const members = $gameParty.battleMembers();
        const rows = BattleManager.getPlayerFormationRows();
        const columns = Math.ceil(members.length / rows);
        const spacingX = 80;
        const spacingY = 60;

        if (this._spriteset && this._spriteset._actorSprites) {
            members.forEach((actor, index) => {
                const row = Math.floor(index / columns);
                const col = index % columns;
                const sprite = this._spriteset._actorSprites[index];
                if (sprite) {
                    const centerX = Graphics.width / 2;
                    const centerY = Graphics.height / 2 + 100;
                    const posX = centerX - ((columns - 1) * spacingX) / 2 + col * spacingX;
                    const posY = centerY - ((rows - 1) * spacingY) / 2 + row * spacingY;
                    sprite.setHome(posX, posY);
                }
            });
        } else {
            console.warn('error');
        }
    };

    Sprite_Actor.prototype.setActorHome = function (index) {
        const baseY = 280;
        const spacingY = 48;
        const spacingX = 32;
        const totalMembers = $gameParty.battleMembers().length;

        if (totalMembers <= 4) {
            this.setHome(600 + index * spacingX, baseY + index * spacingY);
        } else {
            const column = Math.ceil(totalMembers / 4);
            const halfMembers = Math.ceil(totalMembers / column);
            const rowIndex = Math.floor(index / halfMembers);
            const columnIndex = index % halfMembers;
            const minX = 568;
            const maxX = Graphics.width - (spacingX / 2);
            let posX = Math.floor(Math.random() * (maxX - minX + 1)) + minX;
            const posY = baseY + columnIndex * spacingY;
            posX += (rowIndex === 0 ? -((column - 1) * spacingX / 2) : (columnIndex * spacingX));
            this.setHome(posX, posY);
        }
    };

    const _Spriteset_Battle_createActors = Spriteset_Battle.prototype.createActors;
    Spriteset_Battle.prototype.createActors = function () {
        _Spriteset_Battle_createActors.call(this);
        this.refreshActorPositions();
    };

    Spriteset_Battle.prototype.refreshActorPositions = function () {
        const scene = SceneManager._scene;
        if (scene && scene.setupPlayerFormation) {
            scene.setupPlayerFormation();
        }
    };

})();